package cn.wise.sc.cement.business.controller;

import cn.wise.sc.cement.business.entity.NonStandardValue;
import cn.wise.sc.cement.business.model.BaseResponse;
import cn.wise.sc.cement.business.model.PageQuery;
import cn.wise.sc.cement.business.service.INonStandardValueService;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.*;

import javax.servlet.http.HttpServletResponse;

/**
 * <p>
 *  前端控制器
 * </p>
 *
 * @author wlb
 * @since 2020-09-23
 */
@Api(tags="非标产值-非标产值")
@RestController
@RequestMapping("/tcdri/nonestandard_value")
public class NonStandardValueController {
	private static final Logger log = LoggerFactory.getLogger("NoneStandardValueController");

	@Autowired
	private INonStandardValueService inonStandardValueService;

	@ApiOperation(value = "非标产值分页")
	@GetMapping("/getPage")
	public BaseResponse getPage(PageQuery pageQuery,Integer status) {
		try {
			return inonStandardValueService.getPage(pageQuery,status);
		} catch (Exception e) {
			log.debug("非标产值分页列表{}", e);
		}
		return BaseResponse.errorMsg("失败!");
	}

	@ApiOperation(value = "根据非标产值ID查找指定信息")
	@GetMapping("/{id}")
	public BaseResponse getById(@PathVariable Integer id){
		try {
			NonStandardValue e = inonStandardValueService.getById(id);
			if(e == null){
				return BaseResponse.errorMsg("信息错误!");
			}
			return BaseResponse.okData(e);
		}catch (Exception e){
			log.debug("根据非标产值ID查找指定信息{}",e);
		}
		return BaseResponse.errorMsg("失败!");
	}

	@ApiOperation(value = "获取所有已通过的非标产值")
	@GetMapping("/getList")
	public BaseResponse getList() {
		try {
			return inonStandardValueService.getList();
		} catch (Exception e) {
			log.debug("获取所有已通过的非标产值{}", e);
		}
		return BaseResponse.errorMsg("失败!");
	}

	@ApiOperation("非标产值信息导出列表")
	@PostMapping("/exportList")
	public void exportList(Integer id, String filename, String account, String name, HttpServletResponse response) {
		try {
			inonStandardValueService.exportList(id, filename, account, name, response);
		} catch (Exception e) {
			log.debug("非标产值信息导出列表{}", e);
		}
	}
}
