package cn.wise.sc.cement.business.controller;

import cn.wise.sc.cement.business.entity.EntrustReport;
import cn.wise.sc.cement.business.model.BaseResponse;
import cn.wise.sc.cement.business.model.PageQuery;
import cn.wise.sc.cement.business.service.IEntrustService;
import cn.wise.sc.cement.business.util.PageUtil;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.GetMapping;
import org.springframework.web.bind.annotation.PathVariable;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RestController;

import java.util.List;
import java.util.stream.Collectors;

/**
 * @description: Quality
 * @author: qh
 * @create: 2020-09-28 09:49
 **/
@Api(tags = "质量管理")
@Slf4j
@RestController
@RequestMapping("/business/quality")
public class QualityController {

    final
    IEntrustService iEntrustService;

    public QualityController(IEntrustService iEntrustService) {
        this.iEntrustService = iEntrustService;
    }

    @GetMapping("/{entrustId}")
    @ApiOperation("获取报告详情")
    public BaseResponse<com.baomidou.mybatisplus.extension.plugins.pagination.Page<EntrustReport>> getReportDetail(@PathVariable("entrustId") Integer entrustId, PageQuery pageQuery) {

        List<EntrustReport> entrustReports = iEntrustService.getReportDetail(entrustId);

        if (entrustReports.size() != 0) {
            //过滤 id==null 和 projectType != '常规项目'
            List<EntrustReport> list = entrustReports.stream()
                    .filter(arg -> arg.getId() != null && "质量控制".equals(arg.getProjectType()))
                    .collect(Collectors.toList());
            Page<EntrustReport> rts = PageUtil.listConvertToPage(list, pageQuery);
            return BaseResponse.okData(rts);
        }
        return BaseResponse.errorMsg("没找到相关数据!");
    }

}
