package cn.wise.sc.cement.business.service.impl;

import cn.wise.sc.cement.business.entity.NonStandardApply;
import cn.wise.sc.cement.business.mapper.NonStandardApplyMapper;
import cn.wise.sc.cement.business.model.BaseResponse;
import cn.wise.sc.cement.business.model.PageQuery;
import cn.wise.sc.cement.business.model.query.NonStandardApplyQuery;
import cn.wise.sc.cement.business.model.vo.NonStandardApplyVo;
import cn.wise.sc.cement.business.service.INonStandardApplyService;
import cn.wise.sc.cement.business.util.ExcelUtil;
import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.core.metadata.IPage;
import com.baomidou.mybatisplus.core.toolkit.CollectionUtils;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import org.springframework.beans.BeanUtils;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

import javax.annotation.Resource;
import javax.servlet.http.HttpServletResponse;
import java.time.LocalDate;
import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

/**
 * <p>
 * 服务实现类
 * </p>
 *
 * @author wlb
 * @since 2020-09-23
 */
@Service
public class NonStandardApplyServiceImpl extends ServiceImpl<NonStandardApplyMapper, NonStandardApply> implements INonStandardApplyService {
	@Resource
	private NonStandardApplyMapper nonStandardApplyMapper;

	@Override
	public BaseResponse<IPage<NonStandardApplyVo>> getPage (PageQuery pageQuery,Integer status){
		Map<String, Object> params = new HashMap<>();
		params.put("status", status);
		Page<NonStandardApplyVo> page = new Page<>(pageQuery.getPageNo(), pageQuery.getPageSize());
		IPage<NonStandardApplyVo> pages = nonStandardApplyMapper.getPage(page,params);
		return BaseResponse.okData(pages);
	}

	@Override
	public BaseResponse<List<NonStandardApply>> getList() {
		QueryWrapper<NonStandardApply> qw = new QueryWrapper<>();
		qw.ge("status", 0);
		List<NonStandardApply> list = this.list(qw);
		return BaseResponse.okData(list);
	}

	@Override
	@Transactional
	public BaseResponse<NonStandardApply> create(NonStandardApplyQuery query) {
		NonStandardApply cama = new NonStandardApply();
		BeanUtils.copyProperties(query, cama);
		LocalDate st=cama.getStartTime();
		Double a=cama.getReportedHours();
		Double b=a/24+1;
		long c=new Double(b).longValue();
		cama.setStatistical(cama.getStartTime() + "/" + cama.getStartTime().plusDays(c));
		nonStandardApplyMapper.insert(cama);
		return BaseResponse.okData(cama);
	}

	@Override
	public BaseResponse<NonStandardApply> update(NonStandardApplyQuery query) {
		NonStandardApply nsa = new NonStandardApply();
		BeanUtils.copyProperties(query, nsa);
		nsa.setId(query.getId());
		nonStandardApplyMapper.updateById(nsa);
		return BaseResponse.okData(nsa);
	}

	@Override
	public BaseResponse delete(Integer id) {
		NonStandardApply nsa = nonStandardApplyMapper.selectById(id);
		if(nsa == null){
			return BaseResponse.errorMsg("检测依据不存在");
		}
		nonStandardApplyMapper.deleteById(id);
		return BaseResponse.okData("删除成功");
	}

	@Override
	public void exportList(Integer status, String filename, String account, String name, HttpServletResponse response) {
		Map<String, Object> params = new HashMap<>();
		params.put("status", status);
		params.put("name", name);
		params.put("account", account);
		List<Map<String, Object>> list= nonStandardApplyMapper.exportList(params);

		if (!CollectionUtils.isEmpty(list)) {
			Map<String,Object> map=list.get(0);
			String[] headers = new String[map.size()];
			headers[0] = "序号";
			headers[1] = "姓名";
			headers[2] = "账户";
			headers[3] = "起止日期";
			headers[4] = "上报工时";
			headers[5] = "状态";
			headers[6] = "工作类别";
			headers[7] = "开始时间";
			headers[8] = "工作描述";

			List<Object[]> datas = new ArrayList<>(list.size());
			for (Map<String, Object> m : list) {
				Object[] objects = new Object[headers.length];
				for (int j = 0; j < headers.length; j++) {
					String obj = m.get(headers[j]).toString();
					if(j==0){
						obj = obj.split("\\.")[0];
					}
					objects[j] = obj;
				}

				datas.add(objects);
			}
			ExcelUtil.excelExport(
					filename == null || filename.trim().length() <= 0 ? "非标产值申请": filename, headers,
					datas, response);
		}
	}

}
